@@ -3,9 +3,11 @@ from .embedding import StaticGraphEmbedding
 from .graph_conv import GraphConv
 from .temporal_conv import TemporalConv2d, GatedTemporalConv2d
 from .attention import *
+from .dense import Dense
 
 __all__ = [
     'attention',
+    'Dense',
     'GraphConv',
     'TemporalConv2d',
     'GatedTemporalConv2d',
