@@ -1,12 +1,13 @@
 from .link_predictor import LinkPredictor
 from .positional_encoding import PositionalEncoding
 from . import norm, graph_convs
+from .ops import *
 
 __all__ = [
     'graph_convs',
     'norm',
     'LinkPredictor',
     'PositionalEncoding'
-]
+] + ops.__all__
 
 classes = __all__[2:]
