@@ -1,21 +1,23 @@
-from .dense_spatial_conv import SpatialConv, SpatialConvOrderK
+from .dense_graph_conv import DenseGraphConv, DenseGraphConvOrderK
 from .diff_conv import DiffConv
 from .graph_attention import AttentionScores, MultiHeadGraphAttention, GATLayer
 from .gat_conv import GATConv
 from .grin_cell import GRIL
 from .spatio_temporal_att import SpatioTemporalAtt
 from .gated_gn import GatedGraphNetwork
+from .adap_graph_conv import AdaptiveGraphConv
 
 __all__ = [
-    'SpatialConv',
-    'SpatialConvOrderK',
+    'DenseGraphConv',
+    'DenseGraphConvOrderK',
     'DiffConv',
     'MultiHeadGraphAttention',
     'GATConv',
     'GATLayer',
     'GRIL',
     'SpatioTemporalAtt',
-    'GatedGraphNetwork'
+    'GatedGraphNetwork',
+    'AdaptiveGraphConv'
 ]
 
 classes = __all__
