@@ -1,4 +1,4 @@
-from typing import Union, Tuple, List, Optional
+from typing import Union, Tuple, List, Optional, Literal, Type
 
 from numpy import ndarray
 from pandas import DatetimeIndex, PeriodIndex, TimedeltaIndex, DataFrame
@@ -6,6 +6,8 @@ from scipy.sparse import coo_matrix, csr_matrix, csc_matrix
 from torch import Tensor
 from torch_sparse import SparseTensor
 
+Scalar = Union[int, float]
+
 TensArray = Union[Tensor, ndarray]
 OptTensArray = Optional[TensArray]
 
@@ -23,3 +25,8 @@ TemporalIndex = Union[DatetimeIndex, PeriodIndex, TimedeltaIndex]
 
 Index = Union[List, Tuple, TensArray]
 IndexSlice = Union[slice, Index]
+
+FillOptions = Optional[Literal["backfill", "bfill", "ffill", "pad",
+                               "mean", "linear"]]
+
+ModelReturnOptions = Union[Type[Tensor], Type[dict], Type[list]]
